import click

from ..k8s import k8s, common_k8s_options
from ...utils.CronUtils import run_progress_barV2
from ...utils.MessageUtils import info, colored
from ...utils.k8s.PodUtils import list_pods, delete_pod, shell_into_pod, get_pod_logs


@k8s.group(
    help="Kubernetes pod related commands ",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def pod():
    pass


@pod.command()
@common_k8s_options
@click.option("-n", "--namespace", default="default", help="Kubernetes namespace")
@click.option(
    "-r",
    "--records",
    default=10,
    help="Number of records to show",
    required=False,
)
@click.option(
    "-w",
    "--watch",
    help="This will enable continuous running of this command, every given of seconds",
)
def show(context, namespace, records, watch):
    """
    This Lists all Pods in a given namespace, If not passed 'default' namespace will be used

    **Example Usage**:

    .. code-block:: shell

        $ hckr k8s pod show --namespace default

    **Command Reference**:
    """
    if context:
        info(f"Using context: {context}")
    info(f"Listing all Pods in namespace: {namespace}")
    if watch:
        info(
            f"Watch {colored('enabled', 'yellow')}, running this command every {watch} seconds"
        )
        while True:
            list_pods(context, namespace, records)
            run_progress_barV2(int(watch))

    else:
        list_pods(context, namespace, records)


@pod.command()
@click.argument("pod_name")
@common_k8s_options
@click.option("-n", "--namespace", default="default", help="Kubernetes namespace")
def delete(context, namespace, pod_name):
    """
    Delete a pod in given namespace and context (default: current context)

    **Example Usage**:

    .. code-block:: shell

        $ hckr k8s pod delete <POD_NAME> --namespace default

    **Command Reference**:
    """
    if context:
        info(f"Using context: {context}")
    delete_pod(context, namespace, pod_name)


@pod.command()
@click.argument("pod_name")
@common_k8s_options
@click.option("-n", "--namespace", default="default", help="Kubernetes namespace")
@click.option(
    "--container",
    help="Kubernetes container to shell, If not provided hckr try to infer from pod",
)
def shell(context, namespace, pod_name, container):
    """
    Shell into a pod in the given namespace and context (default: current context)

    **Example Usage**:

    .. code-block:: shell

        $ hckr k8s pod shell <POD_NAME> --namespace default

    **Command Reference**:
    """
    if context:
        info(f"Using context: {context}")
    shell_into_pod(context, namespace, pod_name, container)


@pod.command()
@click.argument("pod_name")
@common_k8s_options
@click.option("-n", "--namespace", default="default", help="Kubernetes namespace")
@click.option(
    "--container",
    help="Kubernetes container to check logs, If not provided hckr try to infer from pod",
)
@click.option(
    "-w", "--watch", default=False, is_flag=True, help="Whether to watch/follow logs"
)
def logs(context, namespace, pod_name, container, watch):
    """
    Get logs from a pod in the given namespace and context (default: current context)

    **Example Usage**:

    .. code-block:: shell

        $ hckr k8s pod logs <POD_NAME> --namespace default

    **Command Reference**:
    """
    if context:
        info(f"Using context: {context}")
    get_pod_logs(context, namespace, pod_name, container, watch)



import json
import logging

import click
import fastavro
import pandas as pd
import pyarrow as pa  # type: ignore
import rich
from cron_descriptor import get_description  # type: ignore
from faker import Faker
from pyarrow import parquet as pq  # type: ignore
from rich.panel import Panel

from hckr.utils.DataUtils import safe_faker_method, print_df_as_table, readFile
from hckr.utils.FileUtils import (
    validate_file_extension,
    get_file_format_from_extension,
    FileFormat,
)
from hckr.utils.MessageUtils import success, colored, info, error


@click.group(
    help="data related commands",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def data():
    pass


@data.command()
@click.option(
    "-c",
    "--count",
    default=10,
    help="Number of data entries to generate.",
    required=False,
)
@click.option(
    "-s",
    "--schema",
    type=click.Path(exists=True),
    help="Path to the JSON schema file.",
    required=True,
)
@click.option("-o", "--output", help="Output file path", required=True)
@click.option(
    "-f",
    "--format",
    help=f"Output file format, Options: {FileFormat.validFormats()} [default: Inferred from file extension]",
    required=False,
)
def faker(count, schema, output, format):
    """
    This command Generate fake Data in different file formats

    **Example Usage**:

    * let we have the following schema in format `{"column_name": "faker_provider"}`
    * Please find all faker provider  `here <https://faker.readthedocs.io/en/master/providers.html>`_

    .. code-block:: json
        :caption: schema.json

        {
            "user_name": "name",
            "user_address": "address",
            "user_email": "email"
        }

    * Generate **csv** data using schema

    .. code-block:: shell

        $ hckr data faker -s schema.json -o file.avro

    * Similarly, we can generate **avro** (or any other valid format) data

    .. code-block:: shell

        $ hckr data faker -s schema.json -o file.csv

    * We can also provide data format explicitly, if file extension is not clear

    .. code-block:: shell

        $ hckr data faker -s schema.json -o output -f csv


    **Command Reference**:
    """

    try:
        if not format:
            format = get_file_format_from_extension(output)
            info(
                f"File format is not passed, inferring from output file path {colored(output, 'yellow')}"
            )
            info(f"Format inferred: {colored(format, 'magenta')}")
        else:
            format = format.lower()
        fake = Faker()
        with open(schema, "r") as schema_file:
            schema = json.load(schema_file)

        records = []

        # Adjusted fake data generation
        for _ in range(count):
            entry = {
                key: safe_faker_method(fake, *value.split(":"))
                for key, value in schema.items()
            }
            records.append(entry)

        df = pd.DataFrame(records)

        # this is same as CSV in case of faker as columns are determined by schema
        if format == FileFormat.TXT or format == FileFormat.CSV:
            df.to_csv(output, index=False, quotechar='"', escapechar="\\")
        elif format == FileFormat.JSON:
            df.to_json(output, orient="records")
        elif format == FileFormat.EXCEL:
            validate_file_extension(output, [".xlsx", ".xls"])
            df.to_excel(output, index=False, engine="openpyxl")
        elif format == FileFormat.PARQUET:
            table = pa.Table.from_pandas(df)
            pq.write_table(table, output)
        elif format == FileFormat.AVRO:
            records = df.to_dict(orient="records")
            schema = {
                "doc": "Avro schema for generated data",
                "name": "Root",
                "type": "record",
                "fields": [
                    {"name": str(key), "type": ["null", "string"], "default": None}
                    for key in records[0]
                ],
            }
            parsed_schema = fastavro.parse_schema(schema)
            with open(output, "wb") as out:
                fastavro.writer(out, parsed_schema, records)
        else:
            error(
                f"Invalid file format {str(format)}, Available {FileFormat.validFormats()}"
            )
            exit(1)
        info(
            f"Generating {colored(count, 'magenta')} rows in {colored(format, 'yellow')} format."
        )
        # logging.debug(f"Data head:\n'{df.head()}'")

        print_df_as_table(df)
        success(
            f"Data written to {colored(output, 'magenta')} in {colored(format, 'yellow')} format."
        )

        rich.print(
            Panel(
                output,
                expand=True,
                title="File Output",
            )
        )
    except Exception as e:
        error(f"Some error occurred while generating data\n{e}")


@data.command()
@click.option("-i", "--input", help="Input file to peek into", required=True)
@click.option(
    "-c",
    "--count",
    default=10,
    help="Number of rows to show, [default: 10]",
    required=False,
)
@click.option(
    "-f",
    "--format",
    help="Output file format, if not provided it gets inferred from file extension",
    required=False,
)
def peek(input, count, format):
    """
    This command allows us to peek into top COUNT rows from a file

    **Example Usage**:

    * We can look into an input file by providing **-i** or **-\-input** option
    * It will show top 10 rows ( by default )

    .. code-block:: shell

        $ hckr data peek -i input.avro

    * We can also provide number of rows we want to see using **-c** or **-\-count** option

    .. code-block:: shell

        $ hckr data peek -i input.avro -c 20

    * We can also provide data format explicitly using **-f** or **-\-format** option, if file extension is not clear

    .. code-block:: shell

        $ hckr data peek -i input.data -f csv


    **Command Reference**:
    """
    try:
        info(
            f"Peeking {colored(count, 'magenta')} rows in file {colored(input, 'yellow')}"
        )
        if not format:
            format = get_file_format_from_extension(input)
            info(
                f"File format is not passed, inferring from file path {colored(input, 'yellow')}"
            )
            info(f"Format inferred: {colored(format, 'magenta')}")
        else:
            format = format.lower()

        # Adjusted fake data generation
        df = readFile(format, input)
        # logging.debug(f"Data head:\n'{df.head()}'")
        print_df_as_table(df, count=count)
    except Exception as e:
        error(
            f"Some error occurred while reading data {colored(input,'magenta')} in format {colored(format,'yellow')}\n{e}"
        )



import click


def common_k8s_options(func):
    func = click.option(
        "-c",
        "--context",
        help="Kubernetes context to be used, [default: Current active context]",
    )(func)
    return func


@click.group(
    help="Kubernetes commands",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def k8s():
    pass



from ..k8s import k8s
from ...utils.MessageUtils import info
from ...utils.k8s.K8sUtils import list_contexts


@k8s.group(
    help="Kubernetes context related commands ",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def context():
    pass


@context.command()
def show():
    """
    This Command Lists all contexts available in kube config

    **Example Usage**:

    .. code-block:: shell

        $ hckr k8s show contexts

    **Command Reference**:
    """

    info("Listing all contexts")
    list_contexts()



# SPDX-FileCopyrightText: 2024-present Ashish Patel <ashishpatel0720@gmail.com>
#
# SPDX-License-Identifier: MIT
import logging

import click
from click_repl import register_repl  # type: ignore

from hckr.cli.k8s.context import context
from hckr.cli.k8s.namespace import namespace
from hckr.cli.k8s.pod import pod
from .net import net
from .crypto.fernet import fernet
from .data import data
from .info import info
from .k8s import k8s
from ..__about__ import __version__
from ..cli.cron import cron
from ..cli.crypto import crypto
from ..cli.hash import hash
from ..utils.CliUtils import check_update
from ..utils.MessageUtils import warning

LOGGING_LEVELS = {
    0: logging.NOTSET,
    # 1: logging.ERROR,
    # 2: logging.WARN,
    1: logging.INFO,
    2: logging.DEBUG,
}  #: a mapping of `verbose` option counts to logging levels


# Define a format for the console handler
class Info:
    """An information object to pass data between CLI functions."""

    def __init__(self):  # Note: This object must have an empty constructor.
        """Create a new instance."""
        self.verbose: int = 0


# pass_info is a decorator for functions that pass 'Info' objects.
pass_info = click.make_pass_decorator(Info, ensure=True)


@click.group(
    context_settings={"help_option_names": ["-h", "--help"]},
    invoke_without_command=True,
)
@click.option(
    "--verbose",
    "-v",
    count=True,
    help="Enable verbose output, use -v for INFO and -vv for DEBUG",
)
@click.version_option(version=__version__, prog_name="hckr")
@click.pass_context
@pass_info
def cli(
    _info: Info,
    ctx: click.Context,
    verbose: int,
):
    if verbose > 0:
        logging.basicConfig(
            level=(LOGGING_LEVELS.get(verbose, logging.DEBUG)),
            format="[%(levelname)s]: %(message)s",
        )
        warning(
            f"Verbose logging (LEVEL={'INFO' if verbose == 1 else 'DEBUG'}) is enabled. "
            "Use -v for INFO, -vv for DEBUG logs"
        )
    _info.verbose = verbose
    if ctx.invoked_subcommand is None:
        click.secho("hckr ", fg="magenta", bold=True, nl=False)
        click.secho(f"v{__version__}  ", fg="blue", bold=True, nl=False)
        click.secho(
            f"https://github.com/hckr-cli/hckr\n", fg="green", bold=True, nl=False
        )
        check_update()
        cli(["-h"])
    else:
        check_update()


register_repl(cli)
cli.add_command(info)
cli.add_command(cron)
cli.add_command(hash)
cli.add_command(data)

# we have to add these here otherwise it will be circular imports
# CRYPTO
cli.add_command(crypto)
crypto.add_command(fernet)

# k8s
cli.add_command(k8s)
k8s.add_command(pod)
k8s.add_command(namespace)
k8s.add_command(context)

# NETWORK command
cli.add_command(net)

# implementing this so that if the user just uses `hckr` we show them something
if __name__ == "__main__":
    cli()



from pathlib import Path

import pyarrow as pa  # type: ignore
from click.testing import CliRunner
from pyarrow import parquet as pq  # type: ignore

from hckr.cli.data import peek
from hckr.utils.FileUtils import (
    FileFormat,
)

parent_directory = Path(__file__).parent.parent

INPUT_DIR = parent_directory / "resources" / "data" / "peek"
INPUT_CSV_FILE = INPUT_DIR / "input.csv"


def peekUtil(_format, _count=None):
    runner = CliRunner()
    DEFAULT_COUNT = 10
    FILE = (
        INPUT_DIR / f"input.{'xlsx' if _format == str(FileFormat.EXCEL) else _format}"
    )
    print(f"Reading from file :{FILE}")
    if _count:
        result = runner.invoke(peek, ["-i", FILE, "-c", _count])
        # print(result.output)
        # make sure format is rightly inferred
        # only validate count if we are checking valid case
        assert f"File format is not passed, inferring from file path" in result.output
        assert f"Peeking {_count} rows in file" in result.output
        assert f"Format inferred: {_format}" in result.output
    else:  # if count  not given use default count
        result = runner.invoke(peek, ["-i", FILE])
        # print(result.output)
        assert f"File format is not passed, inferring from file path" in result.output
        assert f"Peeking {DEFAULT_COUNT} rows in file" in result.output
        assert f"Format inferred: {_format}" in result.output

    print(result.output)
    return result


# POSITIVE
def test_data_peek_inferred_format_default_count():
    formats = FileFormat.validFormats()
    for _format in formats:
        print(f"Running for {_format}")
        result = peekUtil(_format=_format)
        assert result.exit_code == 0
        print(f"=" * 50)


def test_data_peek_inferred_format_given_count():
    formats = FileFormat.validFormats()
    for _format in formats:
        print(f"Running for {_format}")
        result = peekUtil(_format=_format, _count=8)
        # print(result.output)
        assert result.exit_code == 0
        assert "showing first 8 rows" in result.output
        print(f"=" * 50)


def test_data_peek_with_large_cols():
    runner = CliRunner()
    result = runner.invoke(peek, ["-i", INPUT_DIR / "Account.csv"])
    print(result.output)
    assert (
        "Data has total 13 rows and 61 columns, showing first 10 rows and 10 columns"
        in result.output
    )


def test_data_peek_with_tsv_format():
    runner = CliRunner()
    result = runner.invoke(peek, ["-i", INPUT_DIR / "TabSep.tsv"])
    print(result.output)
    assert (
        "Data has total 2902 rows and 6 columns, showing first 10 rows" in result.output
    )


def test_data_peek_with_text_format():
    runner = CliRunner()
    result = runner.invoke(peek, ["-i", INPUT_DIR / "simple.text"])
    print(result.output)
    assert (
        "Data has total 29 rows and 1 columns, showing first 10 rows" in result.output
    )


def test_data_peek_with_json_lines():
    runner = CliRunner()

    result = runner.invoke(peek, ["-i", INPUT_DIR / "json-lines.json"])
    print(result.output)
    assert "Data is in JSON Lines format" in result.output
    assert "Data has total 3 rows and 4 columns, showing first 3 rows" in result.output


# NEGATIVE
def test_data_peek_no_input():
    runner = CliRunner()
    result = runner.invoke(peek, [])
    print(result.output)
    assert "Missing option '-i' / '--input'" in result.output


def test_data_peek_with_incompatible_format():
    runner = CliRunner()
    # giving CSV file but Format as AVRO
    result = runner.invoke(peek, ["-i", INPUT_CSV_FILE, "-f", str(FileFormat.AVRO)])
    print(result.output)
    assert "Some error occurred while reading data" in result.output


def test_data_peek_with_invalid_input_file_format():
    runner = CliRunner()
    INVALID_FORMAT = "dvs"
    # giving CSV file but Format as AVRO
    result = runner.invoke(peek, ["-i", INPUT_CSV_FILE, "-f", INVALID_FORMAT])
    print(result.output)
    assert f"""Invalid file format {INVALID_FORMAT}""" in result.output


def test_data_peek_with_input_file_not_found():
    runner = CliRunner()
    # giving CSV file but Format as AVRO
    result = runner.invoke(peek, ["-i", "invalid.csv"])
    print(result.output)
    assert "Some error occurred while reading data" in result.output
    assert "[Errno 2] No such file or directory: 'invalid.csv" in result.output



import logging

from rich import print
import random


def colored(msg, color, bold=True):
    if bold:
        return f"[bold {color}]{msg}[/bold {color}]"
    else:
        return f"[{color}]{msg}[/{color}]"


def success(msg):
    print(f"{success_emoji()} [bold green] {msg}[/bold green]")


def error(msg, color=None):
    if color:
        print(f"{error_emoji()} [bold {color}] {msg}[/bold {color}]")
    else:
        print(f"{error_emoji()} [bold red]{msg}[/bold red]")


def warning(msg, color=None):
    if color:
        print(f"{warn_emoji()} [bold {color}] {msg}[/bold {color}]")
    else:
        print(f"{warn_emoji()} [bold yellow]{msg}[/bold yellow]")


def info(msg, color=None):
    if color:
        print(f"{info_emoji()} [bold {color}] {msg}[/bold {color}]")
    else:
        print(f"{info_emoji()} [bold blue] {msg}[/bold blue]")


def info_emoji():
    emoji = random.choice(["information", "slightly_smiling_face", "smiley_cat"])
    return f":{emoji}:"


def success_emoji():
    emoji = random.choice(
        ["sparkles", "money_with_wings", "white_check_mark", "party_popper", "tada"]
    )
    return f":{emoji}:"


def error_emoji():
    emoji = random.choice(["cross_mark", "face_screaming_in_fear", "scream"])
    return f":{emoji}:"


def warn_emoji():
    emoji = random.choice(["warning", "children_crossing"])
    return f":{emoji}:"


def checkOnlyOnePassed(name1, val1, name2, val2):
    if val1 and val2:
        logging.debug(f"Both values  --{name1} and --{name2} are not None")
        error(f"Please use either --{name1} or --{name2}, can't use both")
        exit(1)
    if not val1 and not val2:
        logging.debug(f"Both values --{name1} and --{name2} are None")
        error(f"Please provide either --{name1} or --{name2}")
        exit(1)



import logging
from pathlib import Path

import click

from hckr.utils.CryptoUtils import (
    encrypt_message,
    checkAndCreateKey,
    encrypt_file_content,
    decrypt_message,
    decrypt_file_content,
)
from hckr.utils.FileUtils import load_file, save_file
from hckr.utils.MessageUtils import (
    success,
    error,
    colored,
    info,
)
from ..crypto import crypto
import rich
from rich.panel import Panel


@crypto.group(
    help="cryptography using fernet",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def fernet():
    pass


@fernet.command()
@click.option(
    "-k",
    "--key",
    type=click.Path(),
    help="Path to the encryption key file.",
    required=True,
)
@click.option("-m", "--message", help="message to be encrypted", required=True)
@click.option(
    "-c",
    "--create-key",
    help="Flat to auto create key if key path doesn't exists",
    default=False,
    is_flag=True,
    required=False,
)
def encrypt(key, message, create_key):
    """
    This command encrypts a TEXT message using **Fernet** key.

    **Example Usage**:

    * Encrypt a message with given key file

    .. code-block:: shell

        $ hckr crypto fernet encrypt -m 'hello'  -k 'key.txt'

    * Encrypt a message and create new key file

    .. code-block:: shell

        $ hckr crypto fernet encrypt -m 'hello'  -k 'key.txt' -c

    **Command Reference**:
    """

    checkAndCreateKey(key, message, create_key)
    new_key = load_file(key)
    encrypted_message = encrypt_message(message, new_key)
    rich.print(
        Panel(
            encrypted_message.decode(),
            expand=True,
            title="Encrypted message",
        )
    )


@fernet.command()
@click.option(
    "-k",
    "--key",
    type=click.Path(),
    help="Path to the encryption key file.",
    required=True,
)
@click.option(
    "-f", "--file", type=click.Path(), help="File to be encrypted", required=True
)
@click.option(
    "-o",
    "--output",
    type=click.Path(),
    help="Output file containing encrypted file",
    required=True,
)
@click.option(
    "-c",
    "--create-key",
    help="Flat to auto create key if key path doesn't exists",
    default=False,
    is_flag=True,
    required=False,
)
def encrypt_file(key, file, output, create_key):
    """
    This command encrypts a file PATH using **Fernet** key and generates output file

    **Example Usage**:

    * Encrypt a file with given key file

    .. code-block:: shell

        $ hckr crypto fernet encrypt -f 'message.txt'  -k 'key.txt' -o 'output.txt'

    * Encrypt a file and create new key file

    .. code-block:: shell

        $ hckr crypto fernet encrypt -f 'message.txt'  -k 'key.txt' -o 'output.txt' -c

    **Command Reference**:
    """
    checkAndCreateKey(key, file, create_key)
    new_key = load_file(key)
    encrypted_content = encrypt_file_content(file, new_key)
    save_file(encrypted_content, output)
    success(f"File encrypted, output written")
    rich.print(
        Panel(
            output,
            expand=True,
            title="File Output",
        )
    )


@fernet.command()
@click.option(
    "-k",
    "--key",
    type=click.Path(),
    help="Path to the encryption key file.",
    required=True,
)
@click.option("-m", "--message", help="Message to be decrypted", required=True)
def decrypt(key, message):
    """
    This command decrypts a TEXT message using **Fernet** key.

    **Example Usage**:

    * Decrypt a message with given key file

    .. code-block:: shell

        $ hckr crypto fernet decrypt -m 'gAAAAABmdVJkaiK1gUihh-T3Z4-_p2qoy6v3NI7uITyXcsOow==' -k 'key.txt'

    **Command Reference**:
    """
    info(
        f"Decrypting '{colored(message, 'magenta')}', Using key file: {colored(key, 'yellow')}"
    )
    if not Path(key).exists():
        error(
            f"Key file path :{key} doesn't exists, \nPlease check and provide correct encryption key path."
        )
        exit(1)

    new_key = load_file(key)
    decrypted_message = decrypt_message(message, new_key)
    rich.print(
        Panel(
            decrypted_message,
            expand=True,
            title="Decrypted message",
        )
    )


@fernet.command()
@click.option(
    "-k",
    "--key",
    type=click.Path(),
    help="Path to the encryption key file.",
    required=True,
)
@click.option(
    "-f", "--file", type=click.Path(), help="File to be decrypted", required=True
)
@click.option(
    "-o",
    "--output",
    type=click.Path(),
    help="Output file containing decrypted file",
    required=True,
)
def decrypt_file(key, file, output):
    """
    This command decrypts a file PATH using **Fernet** key and generates output file

    **Example Usage**:

    * Encrypt a file with given key file

    .. code-block:: shell

        $ hckr crypto fernet decrypt -f 'encrypted_message.txt'  -k 'key.txt' -o 'output.txt'


    **Command Reference**:
    """
    info(
        f"Decrypting '{colored(file, 'magenta')}', Using key file: {colored(key, 'yellow')}"
    )
    if not Path(key).exists():
        error(
            f"Key file path :{key} doesn't exists, \nPlease check and provide correct encryption key path."
        )
        exit(1)

    new_key = load_file(key)
    decrypted_content = decrypt_file_content(file, new_key)
    save_file(decrypted_content, output)
    success(f"File decrypted, output written")
    logging.debug(f"Message head:\n'{decrypted_content.decode()[:100]}'")
    rich.print(
        Panel(
            output,
            expand=True,
            title="File Output",
        )
    )



from pathlib import Path

import pyarrow as pa  # type: ignore
from click.testing import CliRunner
from pyarrow import parquet as pq  # type: ignore

from hckr.cli.data import faker
from hckr.utils.DataUtils import readFile
from hckr.utils.FileUtils import (
    delete_path_if_exists,
    FileFormat,
)

parent_directory = Path(__file__).parent.parent

SCHEMA_FILE = parent_directory / "resources" / "data" / "faker" / "schema.json"
OUTPUT_DIR = parent_directory / "resources" / "data" / "faker" / "output"


# POSITIVE


def faker_test_util_format_count(_format, _count=None):
    runner = CliRunner()
    DEFAULT_COUNT = 10
    # create a output file according to format given
    FILE = (
        OUTPUT_DIR / f"output.{'xlsx' if _format == str(FileFormat.EXCEL) else _format}"
    )

    delete_path_if_exists(FILE)
    if _count:
        result = runner.invoke(faker, ["-s", SCHEMA_FILE, "-o", FILE, "-c", _count])
        # make sure format is rightly inferred
        assert f"Generating {_count} rows in {_format} format" in result.output
        # only validate count if we are checking valid case
        if result.exit_code == 0:
            df = readFile(_format, FILE)
            assert df.shape[0] == _count
    else:  # if count  not given use default count
        result = runner.invoke(faker, ["-s", SCHEMA_FILE, "-o", FILE])
        assert (
            "File format is not passed, inferring from output file path"
            in result.output
        )
        assert f"Format inferred: {_format}" in result.output
        assert (
            f"Generating {DEFAULT_COUNT} rows in {str(_format)} format" in result.output
        )
        if result.exit_code == 0:
            df = readFile(_format, FILE)
            assert df.shape[0] == DEFAULT_COUNT

    print(result.output)
    return result


def test_data_faker_inferred_format():
    formats = FileFormat.validFormats()
    for _format in formats:
        print(f"Running for {_format}")
        result = faker_test_util_format_count(_format=_format)
        assert result.exit_code == 0
        print(f"=" * 50)


def test_data_faker_given_count():
    result = faker_test_util_format_count(_count=20, _format=str(FileFormat.CSV))
    assert result.exit_code == 0


def test_data_faker_given_format():
    result = faker_test_util_format_count(_format=str(FileFormat.CSV))
    assert result.exit_code == 0


def test_data_faker_given_count_given_format():
    result = faker_test_util_format_count(_format=str(FileFormat.CSV), _count=30)
    assert result.exit_code == 0


def test_data_faker_parquet_format():
    result = faker_test_util_format_count(_format=str(FileFormat.PARQUET), _count=30)
    assert result.exit_code == 0


def test_data_faker_avro_format():
    result = faker_test_util_format_count(_format=str(FileFormat.AVRO), _count=30)
    assert result.exit_code == 0


# NEGATIVE
def test_data_faker_no_schema():
    runner = CliRunner()
    result = runner.invoke(faker, [])
    print(result.output)
    assert "Error: Missing option '-s' / '--schema'" in result.output


def test_data_faker_no_output():
    runner = CliRunner()
    result = runner.invoke(faker, ["-s", SCHEMA_FILE])
    print(result.output)
    assert "Error: Missing option '-o' / '--output'" in result.output


def test_data_faker_invalid_output_format():
    runner = CliRunner()
    result = runner.invoke(
        faker, ["-s", SCHEMA_FILE, "-o", OUTPUT_DIR, "-f", "INVALID"]
    )
    print(result.output)
    assert """Invalid file format invalid""" in result.output


def test_data_faker_excel_invalid_file_extension():
    runner = CliRunner()
    result = runner.invoke(faker, ["-s", SCHEMA_FILE, "-o", "invalid.excel"])

    print(result.output)
    assert result.exit_code == 1
    assert (
        """Invalid file extension .excel for file invalid.excel, 
Available extensions"""
        in result.output
    )
    assert """Or Please provide format using -f / --format option""" in result.output


def test_data_faker_invalid_faker_provider():
    INVALID_SCHEMA_FILE = (
        parent_directory / "resources" / "data" / "faker" / "invalid_schema.json"
    )

    runner = CliRunner()
    result = runner.invoke(faker, ["-s", INVALID_SCHEMA_FILE, "-o", "output.csv"])

    print(result.output)
    assert """No such Faker method: credit_card_invalid""" in result.output


def test_data_faker_schema_file_not_found():
    runner = CliRunner()
    result = runner.invoke(faker, ["-s", "not_found.json", "-o", "output.csv"])

    print(result.output)
    assert (
        """Error: Invalid value for '-s' / '--schema': Path 'not_found.json' does not exist."""
        in result.output
    )



import tomli


# this is needed as we want to use hatch.docs env for all doc dependencies
# but readthedocs can't use hatch env and we have to either provide setup.py or requirement.txt
# using this script we are syncing requirements.txt from docs dependencies automatically
# this is build as part of the `make sync` command

def generate_requirements(hatch_env='docs'):
    # Path to your pyproject.toml
    toml_file_path = 'pyproject.toml'

    try:
        # Open and read the pyproject.toml file
        with open(toml_file_path, 'rb') as toml_file:
            toml_data = tomli.load(toml_file)

        # Extract dependencies for the specific Hatch environment
        dependencies = toml_data.get('tool', {}).get('hatch', {}).get('envs', {}).get(hatch_env, {}).get('dependencies',
                                                                                                         [])

        # Write dependencies to requirements.txt
        if dependencies:
            with open('docs/requirements.txt', 'w') as req_file:
                for dependency in dependencies:
                    req_file.write(f"{dependency}\n")
            print("requirements.txt generated successfully.")
        else:
            print("No dependencies found for the specified environment.")

    except FileNotFoundError as e:
        print("The pyproject.toml file was not found.")
        raise e
    except Exception as e:
        print(f"An error occurred: {e}")
        raise e


if __name__ == "__main__":
    generate_requirements()



import logging
import time
import webbrowser

import click

from ..__about__ import __version__
from ..utils.CliUtils import check_update
from ..utils.MessageUtils import error


@click.group(
    help="info commands",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def info():
    pass


# NOTE - keep command() empty to provide docstr
@info.command()
@click.option("--name", help="Say hello to someone", required=False, default="World")
def hello(name):
    """
    Greets a person with the given name.

    This command outputs a greeting message to the person specified by the `name` option.

    **Example Usage**:

    * Say hello

    .. code-block:: shell

        $ hckr info hello
        Hello World!

    * Say hello to *Alice*

    .. code-block:: shell

        $ hckr info hello --name Alice
        Hello Alice!

    **Command Reference**:
    """
    logging.debug("Hello debug")
    logging.info("Hello info")
    click.secho(f"Hello {name}!", fg="green", nl=True)


@info.command()
def version():
    """
    Show information for CLI and useful links.

    **Example usage**:

    .. code-block:: shell

        $ hckr info version
        Version: hckr==VERSION
        Github: https://github.com/hckr-cli/hckr
        PyPi: https://pypi.org/project/hckr/

    **Command Reference**:
    """
    click.secho("Version: hckr", fg="magenta", bold=True, nl=False)
    click.secho(f"=={__version__}  ", fg="blue", bold=False, nl=True)
    check_update(show_no_update=True)
    click.secho(
        f"Github: https://github.com/hckr-cli/hckr", fg="red", bold=True, nl=True
    )
    click.secho(
        f"PyPi: https://pypi.org/project/hckr/", fg="yellow", bold=True, nl=True
    )


@info.command(help="Opens docs for hckr cli in browser")
def docs():
    url = "https://hckr.readthedocs.io/"
    click.secho("Version: hckr", fg="yellow", bold=True, nl=False)
    click.secho(f"=={__version__}  ", fg="blue", bold=False, nl=True)
    click.secho(f"Opening Docs in browser: {url}", fg="red", bold=True, nl=True)
    time.sleep(1)
    try:
        webbrowser.open(url)
    except Exception as e:
        error(f"Some error occured while opening docs in browser\n{e}")



import unittest
from pathlib import Path  # type: ignore
from unittest.mock import patch, MagicMock  # type: ignore

from click.testing import CliRunner

from hckr.cli.k8s.pod import show


class TestK8sPodCLI(unittest.TestCase):

    # note - we have to patch wherever this is used not wherever it is defined
    @patch("hckr.utils.k8s.PodUtils._getApi")
    def test_list_pods(self, mock_core_v1_api):
        runner = CliRunner()

        mock_api = MagicMock()
        mock_api.list_namespaced_pod.return_value = MagicMock(items=[])

        mock_core_v1_api.return_value = (mock_api, "default")

        result = runner.invoke(show)
        print(result.output)
        assert "Listing all Pods in namespace: default" in result.output


# TODO: add tests for delete, logs etc.



from datetime import datetime, timezone

import rich
from kubernetes import client, config  # type: ignore
from rich.panel import Panel

from ..MessageUtils import error, info


def _getApi(context):
    try:
        if context:
            config.load_kube_config(context=context)
            return client.CoreV1Api(), context
        else:
            config.load_kube_config()
            _, currentContext = config.list_kube_config_contexts()
            return client.CoreV1Api(), currentContext["name"]
    except config.ConfigException as e:
        error(f"Error loading kube-config: \n{e}")
        exit(1)


def _human_readable_age(start_time):
    now = datetime.now(timezone.utc)
    delta = now - start_time

    days = delta.days
    hours, remainder = divmod(delta.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)

    if days > 0:
        return f"{days}d{hours}h"
    elif hours > 0:
        return f"{hours}h{minutes}m"
    elif minutes > 0:
        return f"{minutes}m{seconds}s"
    else:
        return f"{seconds}s"


def list_namespaces(context):
    coreApi, currentContext = _getApi(context)
    if context:
        info(f"Using given context: {context}")
    else:
        info(f"Using default context: {currentContext}")

    ret = coreApi.list_namespace()
    rich.print(
        Panel(
            (
                "\n".join(i.metadata.name for i in ret.items)
                if len(ret.items) != 0
                else "NOTHING FOUND"
            ),
            expand=True,
            title=f"Namespaces in context: {currentContext}",
        )
    )


def list_contexts():
    contexts, active_context = config.list_kube_config_contexts()
    if not contexts:
        rich.print("[red]Warning: No contexts found in kube-config file.[/red]")
        return

    context_names = [
        (
            f"[green]{context['name']}[/green] <- [magenta]active[/magenta]"
            if context["name"] == active_context["name"]
            else context["name"]
        )
        for context in contexts
    ]

    rich.print(
        Panel(
            "\n".join(context_names) if context_names else "NOTHING FOUND",
            expand=True,
            title="Contexts",
        )
    )



import time
from datetime import datetime

import click
from croniter import croniter
from rich.progress import track

from hckr.utils.MessageUtils import error


def _get_step(duration):
    if duration < 5:
        step = 1
    elif duration < 20:
        step = 2
    else:
        step = 5
    return step


def run_progress_bar(duration):
    """
    Run a progress bar for the specified duration in seconds.
    """
    step = _get_step(duration)
    with click.progressbar(
        length=duration, label=f"Next job will run in {duration} seconds"
    ) as bar:
        for i in range(0, duration, step):
            time.sleep(step)  # Sleep for a second
            bar.update(step)  # Update the progress bar by one step


# Using Rich's progress bar
def run_progress_barV2(duration):
    """
    Run a progress bar for the specified duration in seconds.
    """
    step = _get_step(duration)
    for _ in track(
        range(0, duration, step),
        description=f"[blue]Next job will run in {duration} seconds",
    ):
        time.sleep(step)  # Sleep for a second


def calculate_sleep_duration(cron_expression):
    # Create a cron iterator based on the cron expression and the current time
    now = datetime.now()

    cron = croniter(cron_expression, now)
    if not cron.is_valid(cron_expression):
        error(f"Invalid cron Expression: {cron_expression}")
        exit(1)
    if len(cron_expression.split(" ")) != 5:
        error(
            f"Invalid cron Expression: {cron_expression}\n cron expression must have 5 places"
        )
        exit(1)
    # Get the next scheduled time
    next_time = cron.get_next(datetime)
    # Calculate the duration in seconds between now and the next scheduled time
    duration_seconds = (next_time - now).total_seconds()
    return round(duration_seconds)



import hashlib
import os
from enum import Enum

from hckr.utils.MessageUtils import colored, info, checkOnlyOnePassed, error, success


class HashType(str, Enum):
    MD5 = ("md5",)
    SHA1 = ("sha1",)
    SHA256 = ("sha256",)
    SHA512 = ("sha512",)


def human_readable_size(size, decimal_places=2):
    # Define the threshold of bytes for units
    for unit in ["B", "KB", "MB", "GB", "TB", "PB"]:
        if size < 1024.0:
            break
        size /= 1024.0
    return f"{size:.{decimal_places}f} {unit}"


def checkFile(_file, _chunk_size):
    if os.path.isdir(_file):
        raise Exception(f"Path is directory, please pass file path: '{_file}'")
    if not os.path.exists(_file):
        raise Exception(f"File not found, Please verify file path: '{_file}'")
    stat_info = os.stat(_file)
    info(
        f"File size: {colored(human_readable_size(stat_info.st_size), 'yellow')}, Using chunk size: {colored(human_readable_size(_chunk_size), 'yellow')}"
    )


def hashStringOrFile(_str, _file, _method, _chunk_size):
    checkOnlyOnePassed("str", _str, "file", _file)
    digest = None
    try:
        m = hashlib.new(_method)
        if _str:
            info(f"Finding hash for string: {_str}")
            m.update(_str.encode())
            digest = m.hexdigest()
        elif _file:
            checkFile(_file, _chunk_size)
            info(f"Finding hash for file: {colored(_file, 'yellow')}")
            with open(_file, "rb") as f:
                # Read and update hash in chunks to avoid using too much memory
                for chunk in iter(lambda: f.read(_chunk_size), b""):
                    m.update(chunk)
                digest = m.hexdigest()
        else:
            error("Invalid option")
    except Exception as e:
        error(f"Some error occurred\n {e}")
    if digest:
        success(f"{_method.upper()}: {digest}")



from click.testing import CliRunner

from hckr.cli.net import speed, ips


def test_net_speed():
    runner = CliRunner()
    result = runner.invoke(speed)
    print(result.output)
    assert "Servers" in result.output and "currently using" in result.output
    assert (
        "Speed test results" in result.output
        and "Download Speed" in result.output
        and "Upload Speed" in result.output
        and "Ping" in result.output
    )


def test_net_ips():
    runner = CliRunner()
    result = runner.invoke(ips)
    # print(result.output)
    assert (
        "Fetching IP addresses" in result.output
        and "IpV4 Addresses" in result.output
        and "IpV4 Addresses" in result.output
    )



from click.testing import CliRunner

from hckr.cli import cli
from hckr.__about__ import __version__


def test_hckr():
    runner = CliRunner()
    result = runner.invoke(cli)
    print(result.output)
    assert result.exit_code == 0
    assert __version__ in result.output
    assert "[OPTIONS] COMMAND [ARGS]..." in result.output
    assert (
        """Options:
  -v, --verbose  Enable verbose output, use -v for INFO and -vv for DEBUG
  --version      Show the version and exit.
  -h, --help     Show this message and exit.

Commands:
  cron    cron commands
  crypto  crypto commands
  data    data related commands
  hash    hash commands
  info    info commands
  k8s     Kubernetes commands
  net     network commands
  repl    Start an interactive shell"""
        in result.output
    )



import logging

import fastavro
import pandas as pd
import rich
from pyarrow import parquet as pq  # type: ignore
from rich.table import Table
import csv
from hckr.utils.FileUtils import FileFormat
from hckr.utils.MessageUtils import error, colored, warning, info

COMMA = ","


def safe_faker_method(faker_instance, method_name, *args):
    """Safely get the Faker method by name and execute it with arguments."""
    if hasattr(faker_instance, method_name):
        method = getattr(faker_instance, method_name)
        return method(*args)
    else:
        raise ValueError(f"No such Faker method: {method_name}")


def print_df_as_table(df, title="Data Sample", count=3):
    MAX_COLS_TO_SHOW = 10
    ROWS_TO_SHOW = min(count, df.shape[0])
    COLS_TO_SHOW = min(MAX_COLS_TO_SHOW, df.shape[1])
    table = Table(
        show_header=True,
        title=title,
        header_style="bold magenta",
        expand=True,
        show_lines=True,  # show line after every row
    )
    table.border_style = "yellow"
    for column in df.columns[:COLS_TO_SHOW]:
        table.add_column(column, no_wrap=False, overflow="fold")
    msg = f"Data has total {colored(df.shape[0], 'yellow')} rows and {colored(df.shape[1], 'yellow')} columns, showing first {colored(ROWS_TO_SHOW, 'yellow')} rows"
    if len(df.columns) > MAX_COLS_TO_SHOW:
        warning(f"{msg} and {colored(COLS_TO_SHOW, 'yellow')} columns")
    else:
        info(msg)
    # Add rows to the table
    for index, row in df.head(count).iterrows():
        # Convert each row to string format, necessary to handle different data types
        table.add_row(*[str(item) for item in row.values[:MAX_COLS_TO_SHOW]])
    rich.print(table)


# Try to read as simple JSON if there is an Error then tries as json lines
def readJSON(_file):
    try:
        return pd.read_json(_file)
    except Exception as e:
        logging.debug(
            f"Error occurred while reading as JSON Object\n{e}\nTrying as JSON Lines"
        )
        df = pd.read_json(_file, lines=True)
        info(f"Data is in {colored('JSON Lines', 'yellow')} format")
        return df


def sniffCsvDelimiter(path):
    try:
        with open(path, mode="r") as csvfile:
            # Sniff the first 1024 bytes to find the dialect
            sample = csvfile.read(2048)
            dialect = csv.Sniffer().sniff(sample)
            logging.debug(f"Delimiter => [{dialect.delimiter}]")
            return dialect.delimiter
    except Exception as e:
        logging.debug(f"Error occured while sniffing delimiter\n{e}")
        return COMMA


def readFile(_format, FILE):
    try:
        if _format == FileFormat.TXT:
            df = pd.read_csv(FILE)
        elif _format == FileFormat.CSV:
            delimiter = sniffCsvDelimiter(FILE)
            df = pd.read_csv(FILE, sep=delimiter)
        elif _format == FileFormat.JSON:
            df = readJSON(FILE)
        elif _format == FileFormat.EXCEL:
            df = pd.read_excel(FILE, engine="openpyxl")
        elif _format == FileFormat.PARQUET:
            df = pq.read_table(FILE).to_pandas()
        elif _format == FileFormat.AVRO:
            with open(FILE, "rb") as f:
                avro_reader = fastavro.reader(f)
                records = [record for record in avro_reader]
            df = pd.DataFrame(records)
        else:
            error(
                f"Invalid file format {colored(str(_format), 'bold yellow')}, Available formats: {colored(FileFormat.validFormats(), 'magenta')}"
            )
            exit(1)
        # print_df_as_table(df, title="Test Read Sample")
        return df
    except Exception as e:
        raise e



import re
from pathlib import Path

from click.testing import CliRunner

from hckr.cli.crypto.fernet import decrypt, decrypt_file

parent_directory = Path(__file__).parent.parent.parent
ENCRYPTED_MESSAGE = "gAAAAABmdVJkaiK1gUihh-T3Z4-_p2qoy6v3NI7uITyXcsOoxOw2pLoAVtA7ZxUwyidHpoywgDBmkxB5N-woLJ0_6jJ09SzG-w=="

ENCRYPTED_MESSAGE_FILE = (
    parent_directory / "resources" / "crypto" / "decrypt" / "encrypted_message.txt"
)
KEY_PATH = parent_directory / "resources" / "crypto" / "key.txt"
OUTPUT_PATH = (
    parent_directory / "resources" / "crypto" / "decrypt" / "decrypted_message.txt"
)


# POSITIVE SCENARIOS
def test_fernet_decrypt_message_with_key():
    runner = CliRunner()
    result = runner.invoke(decrypt, ["-m", ENCRYPTED_MESSAGE, "-k", KEY_PATH])
    print(result.output)
    assert result.exit_code == 0
    assert "Decrypted message" in result.output
    assert "Hello, World!" in result.output


def test_fernet_decrypt_file_with_key():
    runner = CliRunner()
    result = runner.invoke(
        decrypt_file, ["-f", ENCRYPTED_MESSAGE_FILE, "-k", KEY_PATH, "-o", OUTPUT_PATH]
    )
    print(result.output)
    # assert result.exit_code == 0 # TODO: for some reason rich.panel() returning 1 as exit_code
    assert f"""File decrypted, output""" in result.output


def test_fernet_decrypt_no_key_argument():
    runner = CliRunner()
    result = runner.invoke(decrypt, ["-m", ENCRYPTED_MESSAGE])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-k' / '--key'" in result.output


def test_fernet_decrypt_no_message_argument():
    runner = CliRunner()
    result = runner.invoke(decrypt, ["-k", KEY_PATH])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-m' / '--message'" in result.output


def test_fernet_decrypt_no_file_argument():
    runner = CliRunner()
    result = runner.invoke(decrypt_file, ["-k", KEY_PATH])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-f' / '--file'" in result.output


def test_fernet_decrypt_no_output_argument():
    runner = CliRunner()
    result = runner.invoke(decrypt_file, ["-k", KEY_PATH, "-f", ENCRYPTED_MESSAGE_FILE])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-o' / '--output'" in result.output


def test_fernet_decrypt_key_path_doesnt_exist():
    runner = CliRunner()
    INVALID_KEY_PATH = KEY_PATH / "invalid.txt"
    result = runner.invoke(decrypt, ["-m", ENCRYPTED_MESSAGE, "-k", INVALID_KEY_PATH])
    print(result.output)
    assert result.exit_code != 0
    assert f"""Please check and provide correct encryption key path""" in result.output



import logging

import requests
import rich
from packaging import version
from rich.panel import Panel

from .MessageUtils import colored, success, warning
from ..__about__ import __version__
import platform


def check_latest_version():
    current_version = __version__
    try:
        # latest stable version
        response = requests.get("https://pypi.org/pypi/hckr/json")
        latest_version = response.json()["info"]["version"]
        return is_new_version_available(current_version, latest_version), latest_version
    except requests.RequestException:
        return False, current_version


def is_new_version_available(current_version, latest_version):
    current_ver = version.parse(current_version)
    latest_ver = version.parse(latest_version)

    logging.debug(f"current : {current_version}, latest: {latest_version}")

    # Check if the latest version is greater than the current version
    return latest_ver > current_ver


def check_update(show_no_update=False):
    needs_update, latest_version = check_latest_version()
    if needs_update:
        warning(
            f"Info: Update available {colored(__version__, 'magenta')} ->  {colored(latest_version, 'green')}"
        )
        rich.print(
            Panel(
                "pip install --upgrade hckr",
                expand=False,
                title="Run to upgrade using Pip",
            )
        )
        # only show this on MacOs
        if platform.system() == "Darwin":
            rich.print(
                Panel(
                    "brew update && brew upgrade hckr",
                    expand=False,
                    title="Run to upgrade using Homebrew",
                )
            )
    elif show_no_update:
        success(
            f"Success: You are using latest version {colored(__version__, 'magenta')}"
        )



from click.testing import CliRunner

from hckr.cli.info import hello, version
from hckr.__about__ import __version__


def test_info_hello_world():
    runner = CliRunner()
    result = runner.invoke(hello)
    assert result.exit_code == 0
    print(result.output)
    assert "Hello World" in result.output


def test_info_hello_name():
    runner = CliRunner()
    name = "ashish"
    result = runner.invoke(hello, ["--name", name])
    print(result.output)
    assert result.exit_code == 0
    assert f"Hello {name}" in result.output


def test_info_version():
    runner = CliRunner()
    result = runner.invoke(version)
    print(result.output)
    assert result.exit_code == 0
    assert f"Version: hckr=={__version__}" in result.output



import click
import rich
import speedtest  # type: ignore
from rich.panel import Panel
from yaspin import yaspin  # type: ignore

from hckr.utils import NetUtils, MessageUtils
from hckr.utils.NetUtils import get_ip_addresses


@click.group(
    help="network commands",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def net():
    pass


# NOTE - keep command() empty to provide docstr
@net.command()
def speed():
    """
    find your internet speed.


    **Example Usage**:

    .. code-block:: shell

        $ hckr net speed


    **Command Reference**:
    """
    try:
        st = speedtest.Speedtest()
        st.get_best_server()
        servers, current_server = NetUtils.list_servers(st)
        servers_details = [
            (
                f"🔗[green]{server['id']} - {server['sponsor']}, {server['name']}[/green] <- [magenta]currently using[/magenta]"
                if server["id"] == current_server["id"]
                else f"🗄{server['id']} - {server['sponsor']}, {server['name']}"
            )
            for server in servers
        ]
        rich.print(
            Panel(
                "\n".join(servers_details) if servers_details else "NOTHING FOUND",
                expand=False,
                title="Servers",
            )
        )
        ping = st.results.ping
        with yaspin(
            text="Checking internet connection speeds...", color="green", timer=True
        ) as spinner:
            download_speed = (
                st.download(threads=None) / 1_000_000
            )  # Convert from bits/s to Mbps
            upload_speed = (
                st.upload(threads=None) / 1_000_000
            )  # Convert from bits/s to Mbps
            click.echo("\n")
            rich.print(
                Panel(
                    # [magenta]currently using[/magenta]
                    f"[blue]🛜 Ping:[/blue] [green]{ping:.2f} ms[/green]"
                    f"\n[yellow]⬇️ Download Speed:[/yellow] [green]{download_speed:.2f} Mbps[/green]"
                    f"\n[magenta]⬆️ Upload Speed:[/magenta] [green]{upload_speed:.2f} Mbps[/green]",
                    expand=False,
                    title="Speed test results",
                )
            )
            spinner.ok("✔")
    except speedtest.ConfigRetrievalError as e:
        click.echo(
            click.style(f"Failed to retrieve speedtest configuration. {e}", fg="red")
        )
    except speedtest.SpeedtestBestServerFailure as e:
        click.echo(
            click.style(f"Failed to find best server for speedtest. {e}", fg="red")
        )
    except speedtest.SpeedtestException as e:
        click.echo(click.style(f"A speedtest error occurred. {e}", fg="red"))


@net.command()
@click.option(
    "-a",
    "--all",
    default=False,
    is_flag=True,
    help="Whether to show loopback addresses (default: False)",
)
def ips(all):
    """
    find your internet ip addresses.


    **Example Usage**:

    .. code-block:: shell

        $ hckr net ips

    **Command Reference**:
    """
    try:
        MessageUtils.success("Fetching IP addresses...")
        if all:
            MessageUtils.info(
                "-a/--all is passed listing down all IP addresses including loopback addresses"
            )
        ipV4s, ipV6s = get_ip_addresses(all)
        rich.print(
            Panel(
                "\n".join(ipV4s) if ipV4s else "No IpV4 Address FOUND",
                expand=True,
                title="IpV4 Addresses",
            )
        )
        rich.print(
            Panel(
                "\n".join(ipV6s) if ipV4s else "No IpV6 Address FOUND",
                expand=True,
                title="IpV6 Addresses",
            )
        )
    except Exception as e:
        MessageUtils.error(f"Some error occured while fetching IP addresses: {e}")



# SPDX-FileCopyrightText: 2024-present Ashish Patel <ashishpatel0720@gmail.com>
#
# SPDX-License-Identifier: MIT
import sys

if __name__ == "__main__":
    from hckr.cli import cli

    sys.exit(cli())



import logging
import os
from pathlib import Path

from hckr.utils.MessageUtils import error, info, colored, warning

from enum import Enum


class FileFormat(str, Enum):
    TXT = "txt"
    CSV = "csv"
    AVRO = "avro"
    JSON = "json"
    EXCEL = "excel"
    PARQUET = "parquet"
    INVALID = "invalid"

    @staticmethod
    def fileExtToFormat(file_path, file_extension):
        file_type_extension_map = {
            ".txt": FileFormat.TXT,
            ".text": FileFormat.TXT,
            ".csv": FileFormat.CSV,
            ".tsv": FileFormat.CSV,
            ".json": FileFormat.JSON,
            ".xlsx": FileFormat.EXCEL,
            ".xls": FileFormat.EXCEL,
            ".parquet": FileFormat.PARQUET,
            ".avro": FileFormat.AVRO,
        }

        _format = file_type_extension_map.get(
            file_extension.lower(), FileFormat.INVALID
        )
        if _format == FileFormat.INVALID:
            warning("Unable to infer file format from file")
            error(
                f"Invalid file extension {colored(file_extension, 'yellow')} for file {colored(file_path, 'magenta')}, "
                f"\nAvailable extensions {sorted(file_type_extension_map.keys())}\nOr Please provide format using -f / --format option"
            )
            exit(1)
        return _format

    @staticmethod
    def validFormats():
        return [str(x) for x in FileFormat if str(x) != str(FileFormat.INVALID)]

    def __str__(self):
        return self.value


# Save the key to a file
def save_file(content, file_path):
    try:
        path = Path(file_path)
        # creating any intermediate directories if needed
        path.parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, "wb") as key_file:
            key_file.write(content)
    except Exception as e:
        logging.debug(e)
        error(f"Some error occurred while saving {file_path}\n{e}")
        exit(1)


# Load the key from a file
def load_file(file_path):
    checkFileExists(file_path)
    try:
        with open(file_path, "rb") as key_file:
            content = key_file.read()
        return content
    except Exception as e:
        logging.debug(e)
        error(f"Some error occurred while reading {file_path}\n{e}")
        exit(1)


def checkFileExists(file_path):
    logging.debug(f"checkFileExists() invoked with {file_path}")
    path = Path(file_path)
    if path.exists():
        if path.is_file():
            logging.debug("path is a file")
        elif path.is_dir():
            logging.info("path is a directory")
            error(f"Path {path} is a directory")
            exit(1)
    else:
        error(f"File not found, please verify path: {path}")
        exit(1)


def delete_path_if_exists(path):
    """Delete a file or directory if it exists using pathlib."""
    path = Path(path)
    if path.exists():
        if path.is_file():
            logging.info("path is a file, deleted.")
            path.unlink()  # Remove the file
        elif path.is_dir():
            logging.info("path is a directory, removing recursively.")
            path.rmdir()  # Remove the directory, note this directory must be empty
        print(f"Deleted: {path}")
    else:
        logging.info(f"No such path: {path}")


# validate if file extension is one of given
def validate_file_extension(file_path, expected_extensions):
    # Extract the extension from the file path
    _, file_extension = os.path.splitext(file_path)
    # Check if the extension is in the list of expected extensions
    if file_extension.lower() not in expected_extensions:
        error(
            f"Invalid file extension: {file_extension} in file: {file_path}\n allowed: {expected_extensions}"
        )
        exit(1)


# validate if file extension is one of given
def get_file_format_from_extension(file_path):
    if not file_path:
        return FileFormat.INVALID
    # Extract the extension from the file path
    # path = Path(file_path)
    _, file_extension = os.path.splitext(file_path)
    logging.info(f"file extension from file {file_path} is {file_extension}")
    return FileFormat.fileExtToFormat(file_path, file_extension)



import re
from click.testing import CliRunner

from hckr.cli.cron import *


# CRON DESC
def test_cron_desc():
    runner = CliRunner()
    expr = "* 2 3 * *"
    result = runner.invoke(desc, ["--expr", expr])
    # print(result.output)
    assert (
        f"Every minute, between 02:00 AM and 02:59 AM, on day 3 of the month"
        in result.output
    )


def test_cron_desc_invalid():
    runner = CliRunner()
    expr = "* a 3 * *"  # invalid command
    result = runner.invoke(desc, ["--expr", expr])
    # print(result.output)
    assert f"Invalid cron expression" in result.output


# CRON RUN POSITIVE


def test_cron_run_seconds():
    runner = CliRunner()
    expr = "*/1 * * * *"  # invalid command
    cmd = "echo 'hello world!'"
    result = runner.invoke(run, ["--seconds", "2", "--cmd", cmd, "--timeout", "3"])
    print(result.output)
    assert "Output [3/3]" in result.output
    assert "hello world!" in result.output
    assert "Running command: echo 'hello world!', Every 2 seconds" in result.output


def test_cron_run_expr():
    runner = CliRunner()
    expr = "*/1 * * * *"  # every minute
    cmd = "echo 'hello world!'"
    result = runner.invoke(run, ["--expr", expr, "--cmd", cmd, "--timeout", "1"])
    print(result.output)
    assert "Output [1/1]" in result.output
    assert "hello world!" in result.output
    assert "Running command: echo 'hello world!', Every minute" in result.output


# CRON RUN INVALID


def test_cron_run_invalid_cmd():
    runner = CliRunner()
    expr = "*/1 * * * *"  # invalid command
    cmd = "lsdf"
    pattern = re.compile(r"lsdf:.*not found")
    result = runner.invoke(run, ["--seconds", "5", "--cmd", cmd])
    assert "Error in command execution" in result.output
    # assert "not found" in result.output
    print(result.output)
    assert pattern.search(result.output), "Pattern not found"


def test_cron_run_invalid_expr():
    runner = CliRunner()
    expr = "*/1 * * * * *"  # invalid command
    cmd = "ls -la"
    result = runner.invoke(run, ["--expr", expr, "--cmd", cmd])
    assert "Invalid cron Expression: */1 * * * * *" in result.output
    assert "cron expression must have 5 places" in result.output


def test_cron_run_no_schedule():
    runner = CliRunner()
    cmd = "echo 'hello world!'"
    result = runner.invoke(run, ["--cmd", cmd])
    print(result.output)
    assert "Please provide either --seconds or --expr" in result.output


def test_cron_run_both_schedule():
    runner = CliRunner()
    cmd = "echo 'hello world!'"
    expr = "* 2 3 * *"

    result = runner.invoke(run, ["--cmd", cmd, "--expr", expr, "--seconds", "10"])
    print(result.output)
    assert result.exit_code != 0
    assert "Please use either --seconds or --expr, can't use both" in result.output



from ..k8s import k8s, common_k8s_options
from ...utils.k8s.K8sUtils import list_namespaces


@k8s.group(
    help="Kubernetes namespace related commands ",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def namespace():
    pass


@common_k8s_options
@namespace.command()
def show(context):
    """
    This Lists all namespaces available in given namespace, if not passed default context is used

    **Example Usage**:

    .. code-block:: shell

        $ hckr k8s show namespaces --context default

    **Command Reference**:
    """
    # info("Listing all namespaces")
    list_namespaces(context)



import logging
import subprocess

import rich
from cron_descriptor import get_description  # type: ignore
from rich.panel import Panel
from ..utils.CronUtils import calculate_sleep_duration, run_progress_barV2

# from ..utils.MessageUtils import *
import click
from ..utils.MessageUtils import success, error, info, checkOnlyOnePassed, colored


@click.group(
    help="cron commands",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def cron():
    pass


@click.option("-e", "--expr", help="Cron expression", required=True)
@cron.command(help="describe the cron command in human readable terms")
def desc(expr):
    info(f"Describing cron expression: {colored(expr, 'magenta')}")
    try:
        description = get_description(expr)
        success(description)
    except Exception as e:
        logging.debug(
            "Some exception occurred while getting description from Croniter."
        )
        error(f"Invalid cron expression\n '{e}'")


@click.option(
    "-c", "--cmd", help="Command to run as per cron expression", required=True
)
@click.option("-e", "--expr", help="Cron expression", required=False, default=None)
@click.option(
    "-s",
    "--seconds",
    help="Interval in number of seconds to run command",
    required=False,
    default=None,
)
@click.option(
    "-t",
    "--timeout",
    help="Max number of times this command should run",
    required=False,
    default=100,
)
@cron.command(help="run a command using given cron expression")
def run(cmd, expr, seconds, timeout):
    checkOnlyOnePassed("seconds", seconds, "expr", expr)
    try:
        if expr:
            description = get_description(expr)
            info(
                f"Running command: {colored(cmd, 'yellow')}, {colored(description, 'blue')}"
            )
        elif seconds:
            info(
                f"Running command: {colored(cmd, 'yellow')}, {colored(f'Every {seconds} seconds', 'blue')}"
            )
        i = 1
        while i <= timeout:
            result = subprocess.run(cmd, shell=True, text=True, capture_output=True)
            if result.returncode != 0:
                raise Exception(f"Error in command execution {result.stderr}")
            rich.print(
                Panel(
                    result.stdout,
                    expand=True,
                    subtitle="End",
                    title=f"Output [{i}/{timeout}]",
                )
            )
            # rich.print(result.stdout)
            sleep_duration = calculate_sleep_duration(expr) if expr else int(seconds)
            run_progress_barV2(sleep_duration)
            i += 1
    except Exception as e:
        error(f"{e}")
        # raise e



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'hckr'
copyright = '2024, Opensource'
author = 'Ashish'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx_click',
    'notfound.extension',
    'sphinx_docsearch',
    'sphinx_copybutton',
]

docsearch_app_id = "UM5HRVXATR"
docsearch_api_key = "21a390a684536b73f0aee9a20c708c4b"
docsearch_index_name = "hckr"

templates_path = ['_templates']
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

# html_theme = 'alabaster'
html_theme = 'shibuya'
html_static_path = ['_static']
# favicons
html_favicon = '_static/favicon.ico'

# TODO: enhance it https://shibuya.lepture.com/install/
html_theme_options = {
    "color_mode": "light",  # light or dark
    "github_url": "https://github.com/hckr-cli/hckr",
    "discussion_url": "https://github.com/hckr-cli/hckr/discussions",
    "globaltoc_expand_depth": 1,
    "nav_links": [
        {
            "title": "pypi",
            "url": "https://pypi.org/project/hckr/",
            "external": "true"
        },
        {
            "title": "Join our Slack",
            "url": "https://join.slack.com/t/hckr-cli/shared_invite/zt-2kpwz39dw-CIVDcv1nGhEnQNLumJxJDQ",
            "external": "true"
        },
    ],
    "light_logo": "_static/hckr-logo-black.png",
    "dark_logo": "_static/hckr-logo-white.png",
    # "accent_color": "red",
    "announcement": """<div style="text-align: center;">
        <div style="display: inline-block;">
         Please support the project by starring it on <a href='https://github.com/hckr-cli/hckr/' target="_blank">Github</a>
        </div>
    </div>""",
}

html_context = {
    "source_type": "github",
    "source_user": "hckr-cli",
    "source_repo": "hckr",
    "source_version": "main",  # Optional
    "source_docs_path": "/docs/source/",  # Optional
}



from pathlib import Path
from typing import Any

from click.testing import CliRunner
from hckr.cli.hash import sha1, sha256, sha512, md5
from hckr.utils.HashUtils import HashType

current_directory = Path(__file__).parent
HASH_STRING = "Nobody inspects the spammish repetition\n"
HASH_FILE = current_directory / "resources" / "hash" / "test.txt"
EXPECTED_HASHES = {
    "md5": "532027503f1149e968f8465a1b15b62f",
    "sha1": "55cd84c899a3d9f41d5a4d1be4ac57b37551db00",
    "sha256": "a429ddcbe4d2ba357a8c72827f1c9beb878c42081df360c197678263915f3372",
    "sha512": "de60537b3bde7838b3e6763cc94180f0525d17c0335fc2a513eb067c9db367f712a045fa35a6b19c2c4ffd2b81c3091944d94131908daef8a8b8e8c475a56792",
}


def _get_hash_command(method) -> Any:
    if method == HashType.SHA1:
        return sha1
    elif method == HashType.SHA256:
        return sha256
    elif method == HashType.MD5:
        return md5
    elif method == HashType.SHA512:
        return sha512
    else:
        raise Exception(f"Invalid Hash method {method}")


# POSITIVE CASES
def test_string_hashes():
    runner = CliRunner()
    for hash_type in EXPECTED_HASHES:
        print(f"\n{'-' * 50}\n")
        result = runner.invoke(_get_hash_command(hash_type), ["--string", HASH_STRING])
        print(result.output)
        assert result.exit_code == 0
        assert EXPECTED_HASHES[hash_type] in result.output.replace("\n", "")


def test_file_hashes():
    runner = CliRunner()
    for hash_type in EXPECTED_HASHES:
        print(f"\n{'-' * 50}\n")
        result = runner.invoke(_get_hash_command(hash_type), ["--file", HASH_FILE])
        print(result.output)
        assert result.exit_code == 0
        assert "Using chunk size: 4.00 KB" in result.output
        assert EXPECTED_HASHES[hash_type] in result.output.replace("\n", "")


def test_file_hash_with_chunk_size():
    runner = CliRunner()
    # check for all the types
    for hash_type in EXPECTED_HASHES:
        print(f"\n{'-' * 50}\n")
        result = runner.invoke(
            _get_hash_command(hash_type), ["--file", HASH_FILE, "--chunk-size", 1024]
        )
        print(f"{hash_type}: {result.output}")
        assert result.exit_code == 0
        assert "Using chunk size: 1.00 KB" in result.output
        assert EXPECTED_HASHES[hash_type] in result.output.replace("\n", "")


def test_file_hash_invalid_path():
    runner = CliRunner()
    # check for all the types
    for hash_type in EXPECTED_HASHES:
        print(f"\n{'-' * 50}\n")
        result = runner.invoke(
            _get_hash_command(hash_type), ["--file", "/invalid/path/file"]
        )
        print(f"{hash_type}: {result.output}")
        assert result.exit_code == 0
        assert (
            "Some error occurred File not found, Please verify file path"
            in result.output.replace("\n", "")
        )


def test_file_hash_is_dir():
    runner = CliRunner()
    # check for all the types
    for hash_type in EXPECTED_HASHES:
        print(f"\n{'-' * 50}\n")
        result = runner.invoke(
            _get_hash_command(hash_type), ["--file", current_directory]
        )
        print(f"{hash_type}: {result.output}")
        assert result.exit_code == 0
        assert "Path is directory, please pass file path" in result.output.replace(
            "\n", ""
        )



import unittest
from pathlib import Path  # type: ignore
from unittest.mock import patch, MagicMock  # type: ignore

from click.testing import CliRunner

from hckr.cli.k8s.context import show

parent_directory = Path(__file__).parent.parent


class TestK8sContextCLI(unittest.TestCase):
    @patch("kubernetes.config.list_kube_config_contexts")
    def test_list_contexts(self, mock_config_list_context):
        runner = CliRunner()

        # mocking
        contexts = [
            {"name": "dev"},
            {"name": "prod"},
        ]
        active_context = {"name": "dev"}
        mock_config_list_context.return_value = (contexts, active_context)

        result = runner.invoke(show)
        print(result.output)
        # self.assertEqual(result.exit_code, 0)
        assert "Listing all contexts" in result.output
        assert "dev <- active" in result.output
        assert "prod" in result.output



from pathlib import Path

from cryptography.fernet import Fernet

from hckr.utils.MessageUtils import warning, colored, error, info
from .FileUtils import save_file, load_file


# Generate a key and instantiate a Fernet instance
def generate_key():
    return Fernet.generate_key()


# Encrypt a message
def encrypt_message(message, key):
    f = Fernet(key)
    encrypted_message = f.encrypt(message.encode())
    return encrypted_message


# Encrypt a message
def encrypt_file_content(file_path, key):
    f = Fernet(key)
    with open(file_path, "rb") as file:
        original = file.read()
    encrypted = f.encrypt(original)
    return encrypted


# Decrypt a message
def decrypt_message(encrypted_message, key):
    f = Fernet(key)
    decrypted_message = f.decrypt(encrypted_message).decode()
    return decrypted_message


def decrypt_file_content(file_path, key):
    f = Fernet(key)
    with open(file_path, "rb") as file:
        original = file.read()
    decrypted_message = f.decrypt(original)
    return decrypted_message


def checkAndCreateKey(key, content, create_key):
    if not Path(key).exists():
        if create_key:
            warning(
                f"Key file does not exist, Flag [ -c / --create-key ] passed\n Generating a new key: {colored(key, 'magenta')}"
            )
            new_key = generate_key()
            save_file(new_key, key)
        else:
            error(
                f"Key file path :{key} doesn't exists, \nPlease provide -c / --create-key if you want to create one."
            )
            exit(1)
    else:
        info(
            f"Encrypting '{colored(content, 'magenta')}', Using key file: {colored(key, 'yellow')}"
        )



from pathlib import Path

from click.testing import CliRunner

from hckr.cli.crypto.fernet import encrypt, encrypt_file
from hckr.utils.FileUtils import delete_path_if_exists

parent_directory = Path(__file__).parent.parent.parent
MESSAGE = "Hello, World!"

KEY_PATH = parent_directory / "resources" / "crypto" / "key.txt"
MESSAGE_FILE = parent_directory / "resources" / "crypto" / "encrypt" / "message.txt"
OUTPUT_PATH = parent_directory / "resources" / "crypto" / "encrypt" / "output.txt"
NEW_KEY_PATH = parent_directory / "resources" / "crypto" / "encrypt" / "new_key.txt"


# POSITIVE SCENARIOS
def test_fernet_encrypt_message_with_key():
    runner = CliRunner()
    result = runner.invoke(encrypt, ["-m", MESSAGE, "-k", KEY_PATH, "-c"])
    print(result.output)
    # assert result.exit_code == 0 # TODO: for some reason rich.panel() returning 1 as exit_code
    assert (
        f"""Encrypting 'Hello, World!', Using key file: 
{KEY_PATH}"""
        in result.output
    )
    assert "Encrypted message" in result.output


def test_fernet_encrypt_file_with_key():
    runner = CliRunner()
    result = runner.invoke(
        encrypt_file, ["-f", MESSAGE_FILE, "-k", KEY_PATH, "-o", OUTPUT_PATH]
    )
    print(result.output)
    # assert result.exit_code == 0 # TODO: for some reason rich.panel() returning 1 as exit_code
    assert """File encrypted, output written""" in result.output


def test_fernet_encrypt_key_path_doesnt_exist_create_key():
    runner = CliRunner()
    delete_path_if_exists(NEW_KEY_PATH)  # delete the path to achieve this behaviour
    result = runner.invoke(encrypt, ["-m", MESSAGE, "-k", NEW_KEY_PATH, "-c"])
    delete_path_if_exists(NEW_KEY_PATH)  # delete the path to achieve this behaviour
    print(result.output)
    assert (
        result.exit_code == 0
    )  # TODO: for some reason rich.panel() returning 1 as exit_code
    assert (
        """Key file does not exist, Flag [ -c / --create-key ] passed
 Generating a new key"""
        in result.output
    )
    assert "Encrypted message" in result.output


# NEGATIVE SCENARIOS
def test_fernet_encrypt_no_key_argument():
    runner = CliRunner()
    result = runner.invoke(encrypt, ["-m", MESSAGE])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-k' / '--key'" in result.output


def test_fernet_encrypt_no_message_argument():
    runner = CliRunner()
    result = runner.invoke(encrypt, ["-k", KEY_PATH])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-m' / '--message'" in result.output


def test_fernet_encrypt_no_file_argument():
    runner = CliRunner()
    result = runner.invoke(encrypt_file, ["-k", KEY_PATH])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-f' / '--file'" in result.output


def test_fernet_encrypt_no_output_argument():
    runner = CliRunner()
    result = runner.invoke(encrypt_file, ["-k", KEY_PATH, "-f", MESSAGE_FILE])
    print(result.output)
    assert result.exit_code != 0
    assert "Error: Missing option '-o' / '--output'" in result.output


def test_fernet_encrypt_key_path_doesnt_exist():
    runner = CliRunner()
    INVALID_KEY_PATH = KEY_PATH / "invalid.txt"
    result = runner.invoke(encrypt, ["-m", MESSAGE, "-k", INVALID_KEY_PATH])
    print(result.output)
    assert result.exit_code != 0
    assert (
        f"""doesn't exists, 
Please provide -c / --create-key if you want to create one."""
        in result.output
    )



import logging
import re
import sys
import threading

import pandas as pd
import rich
from kubernetes import stream, watch  # type: ignore
from kubernetes.client.exceptions import ApiException  # type: ignore
from yaspin import yaspin  # type: ignore

from hckr.utils.DataUtils import print_df_as_table
from hckr.utils.MessageUtils import error, info, colored, warning, success
from hckr.utils.k8s.K8sUtils import _getApi, _human_readable_age
import logging


def _getContainerName(coreApi, namespace, pod_name):
    pod = coreApi.read_namespaced_pod(name=pod_name, namespace=namespace)
    container_names = [container.name for container in pod.spec.containers]
    if container_names:
        if len(container_names) > 1:
            error(f"Error: Multiple containers found for pod {pod_name}")
            warning(
                f"Please specify one using --container from the following containers: [magenta]{container_names}"
            )
            exit(0)
        else:
            return container_names[0]
    else:
        error(f"No containers found in pod [yellow]{pod}[/yellow]")


def list_pods(context, namespace, count):
    with yaspin(
        text=f"Fetching pods for namespace {namespace}...", color="green", timer=True
    ) as spinner:
        coreApi, currentContext = _getApi(context)
        ret = coreApi.list_namespaced_pod(namespace)
        sorted_pods = sorted(
            ret.items, key=lambda pod: pod.metadata.creation_timestamp, reverse=True
        )
        pods_info = []
        for pod in sorted_pods:  # get last count values
            containers = pod.spec.containers
            container_images = [container.image for container in containers]
            pods_info.append(
                {
                    "Name": pod.metadata.name,
                    "Status": pod.status.phase,
                    # "Start Time": pod.status.start_time,
                    "Age (↑)": _human_readable_age(pod.metadata.creation_timestamp),
                    # TODO: for some issue .metadata.creationTimestamp is different in kubectl and here
                    "Images": ", ".join(container_images),
                    "Containers": ", ".join(
                        [container.name for container in containers]
                    ),
                }
            )
        df = pd.DataFrame(pods_info)
        spinner.ok("✔")

    print_df_as_table(
        df,
        title=f"Pods in context: {currentContext}, namespace: {namespace}",
        count=count,
    )


def delete_pod(context, namespace, pod_name):
    coreApi, currentContext = _getApi(context)
    try:
        info(
            f"Deleting pod {colored(pod_name, 'magenta')} in context: {colored(currentContext, 'yellow')}, namespace: {colored(namespace, 'yellow')}"
        )
        coreApi.delete_namespaced_pod(name=pod_name, namespace=namespace)
        info(f"Pod {colored(pod_name, 'green')} deleted")
    except ApiException as e:
        error(f"Error while delete pod: {colored(pod_name, 'yellow')}: {e.reason}")
    except Exception as e:
        error(f"Error occurred\n {type(e)}")


def shell_into_pod(context, namespace, pod_name, container):
    coreApi, currentContext = _getApi(context)

    def read_stdin():
        try:
            while resp.is_open():
                data = sys.stdin.read(1)
                if data:
                    if resp.is_open():
                        resp.write_stdin(data)
        except KeyboardInterrupt as e:
            logging.info(f"Closing web socket: {e}")
            success("Session closed by user.")
        except EOFError as e:
            logging.info(f"End of file occurred: {e}")
            error(f"End of file occurred\n{e}")

    info(
        f"Starting shell into pod {colored(pod_name, 'magenta')} in context: {colored(currentContext, 'yellow')}, namespace: {colored(namespace, 'yellow')}"
    )
    try:
        if not container:
            logging.info(
                "container is not defined, trying to get container name from pod"
            )
            container = _getContainerName(coreApi, namespace, pod_name)
        exec_command = ["/bin/sh"]

        with yaspin(
            text=f"Connecting to {container} in pod {pod_name}...",
            color="green",
            timer=True,
        ) as spinner:
            resp = stream.stream(
                coreApi.connect_get_namespaced_pod_exec,
                name=pod_name,
                namespace=namespace,
                command=exec_command,
                container=container,
                stderr=True,
                stdin=True,
                stdout=True,
                tty=True,
                _preload_content=False,
                _request_timeout=10,  # time out for connection to container
            )
            spinner.ok("✔")
        info(
            f"You are now in the {colored(container, 'magenta')} shell. Type 'exit' or press 'CTRL-C' to end the session.\n\n"
        )
        threading.Thread(target=read_stdin).start()  # to start input thread
        try:
            while resp.is_open():
                resp.update(timeout=1)
                if resp.peek_stdout():
                    output = resp.read_stdout()
                    sys.stdout.write(output)
                    sys.stdout.flush()
                if resp.peek_stderr():
                    error_output = resp.read_stderr()
                    sys.stderr.write(error_output)
                    sys.stderr.flush()
        except KeyboardInterrupt:
            rich.print("\n[green]Session closed by user.")
        finally:
            if resp.is_open():
                logging.info("Closing web socket")
                resp.close()
    except ApiException as e:
        error(f"Error connection to pod: {colored(pod_name, 'yellow')}\n {e.reason}")
    except Exception as e:
        error(f"Unexpected error occurred: {e}")


def get_pod_logs(context, namespace, pod_name, container, follow):
    coreApi, currentContext = _getApi(context)
    tail_lines = 10

    info(
        f"Getting logs from pod {colored(pod_name, 'magenta')}, container: {container} in context: {colored(currentContext, 'yellow')}, namespace: {colored(namespace, 'yellow')}"
    )
    if follow:
        info("watch is enabled")
    else:
        info("watch is disabled")
    if not container:
        logging.info("container is not defined, trying to get container name from pod")
        container = _getContainerName(coreApi, namespace, pod_name)
        logging.info(f"container is {container}")

    try:
        # Set up streaming of logs if follow is True
        if follow:
            w = watch.Watch()
            try:
                for log_entry in w.stream(
                    coreApi.read_namespaced_pod_log,
                    name=pod_name,
                    namespace=namespace,
                    container=container,
                    tail_lines=tail_lines,
                ):
                    print(log_entry)
            finally:
                w.stop()
        else:
            # Retrieve logs without following (static)
            logs = coreApi.read_namespaced_pod_log(
                name=pod_name,
                namespace=namespace,
                container=container,
                follow=follow,
                tail_lines=tail_lines,
            )
            print(logs)
    except ApiException as e:
        error(f"An exception occurred while reading log: {e}")
        return None



import unittest
from pathlib import Path  # type: ignore
from unittest.mock import patch, MagicMock  # type: ignore

from click.testing import CliRunner

from hckr.cli.k8s.namespace import show


class TestK8sNamespaceCLI(unittest.TestCase):
    @patch("hckr.utils.k8s.K8sUtils._getApi")
    def test_list_namespaces_default_context(self, mock_core_v1_api):
        runner = CliRunner()

        # Create namespace mock objects with correct return values for the name property
        namespace1 = MagicMock()
        namespace1.metadata.name = "test_namespace"

        # Set up the MagicMock for the API client
        mock_api = MagicMock()
        mock_api.list_namespace.return_value = MagicMock(items=[namespace1])

        mock_core_v1_api.return_value = (mock_api, "default")

        result = runner.invoke(show)
        print(result.output)
        assert "Using default context: default" in result.output
        assert "test_namespace" in result.output

    @patch("hckr.utils.k8s.K8sUtils._getApi")
    def test_list_namespaces_given_context(self, mock_core_v1_api):
        runner = CliRunner()
        mock_api = MagicMock()
        mock_api.list_namespace.return_value = []
        mock_core_v1_api.return_value = (mock_api, "default")

        result = runner.invoke(show, ["--context", "mycontext"])
        print(result.output)
        assert "Using given context: mycontext" in result.output



import click
from cron_descriptor import get_description  # type: ignore
from hckr.utils.HashUtils import hashStringOrFile, HashType


@click.group(
    help="hash commands",
    context_settings={"help_option_names": ["-h", "--help"]},
)
def hash():
    pass


def common_hash_options(func):
    func = click.option("-s", "--string", help="String to be hashed", required=False)(
        func
    )
    func = click.option(
        "-f", "--file", type=click.Path(), help="File to be hashed", required=False
    )(func)
    func = click.option(
        "-c",
        "--chunk-size",
        help="Size of chunks for file hash",
        required=False,
        default=4096,
    )(func)
    return func


@common_hash_options
@hash.command(help="hash using MD5 algorithm")
def md5(string, file, chunk_size):
    hashStringOrFile(string, file, HashType.MD5, chunk_size)


@common_hash_options
@hash.command(help="hash using SHA1 algorithm")
def sha1(string, file, chunk_size):
    hashStringOrFile(string, file, HashType.SHA1, chunk_size)


@common_hash_options
@hash.command(help="hash using SHA256 algorithm")
def sha256(string, file, chunk_size):
    hashStringOrFile(string, file, HashType.SHA256, chunk_size)


@common_hash_options
@hash.command(help="hash using SHA512 algorithm")
def sha512(string, file, chunk_size):
    hashStringOrFile(string, file, HashType.SHA512, chunk_size)



import socket


def list_servers(st):
    servers = st.servers
    server_data = []
    for _, server_list in servers.items():
        for server in server_list:
            server_data.append(server)
    return server_data, st.results.server


def get_ip_addresses(all):
    ip_addresses = {"IPv4": [], "IPv6": []}
    for family, _type, _proto, _canonname, sockaddr in socket.getaddrinfo(
        socket.gethostname(), None
    ):
        if family == socket.AF_INET:  # IPv4
            if all or "127." not in sockaddr[0]:  # Skip loopback addresses for IPv4
                ip_addresses["IPv4"].append(sockaddr[0])
        elif family == socket.AF_INET6 and (all or not sockaddr[0].startswith("::1")):
            ip_addresses["IPv6"].append(sockaddr[0])
    return set(ip_addresses["IPv4"]), set(ip_addresses["IPv6"])
